Class: Fastlane::Actions::FirebaseAppDistributionAction
- Inherits:
-
Action
- Object
- Action
- Fastlane::Actions::FirebaseAppDistributionAction
- Extended by:
- Fastlane::Auth::FirebaseAppDistributionAuthClient, Helper::FirebaseAppDistributionHelper
- Defined in:
- lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb
Overview
rubocop:disable Metrics/ClassLength
Constant Summary collapse
- DEFAULT_UPLOAD_TIMEOUT_SECONDS =
300- DEFAULT_UPLOAD_RETRIES =
3- UPLOAD_MAX_POLLING_RETRIES =
60- UPLOAD_POLLING_INTERVAL_SECONDS =
5- TEST_MAX_POLLING_RETRIES =
40- TEST_POLLING_INTERVAL_SECONDS =
30
Constants included from Fastlane::Auth::FirebaseAppDistributionAuthClient
Fastlane::Auth::FirebaseAppDistributionAuthClient::CLIENT_ID, Fastlane::Auth::FirebaseAppDistributionAuthClient::CLIENT_SECRET, Fastlane::Auth::FirebaseAppDistributionAuthClient::REDACTION_CHARACTER, Fastlane::Auth::FirebaseAppDistributionAuthClient::REDACTION_EXPOSED_LENGTH, Fastlane::Auth::FirebaseAppDistributionAuthClient::SCOPE, Fastlane::Auth::FirebaseAppDistributionAuthClient::TOKEN_CREDENTIAL_URI
Class Method Summary collapse
- .aab_certs_included?(test_certificate) ⇒ Boolean
- .aab_info_name(app_name) ⇒ Object
- .authors ⇒ Object
- .available_options ⇒ Object
- .create_release_test(alpha_client, release_name, device_executions, login_credential, test_case_name = nil) ⇒ Object
- .description ⇒ Object
-
.details ⇒ Object
supports markdown.
- .device_to_s(device) ⇒ Object
- .distribute_release(client, release, request) ⇒ Object
- .example_code ⇒ Object
- .extract_release(operation) ⇒ Object
- .get_aab_info(client, app_name) ⇒ Object
-
.get_binary_path(platform, params) ⇒ Object
rubocop:disable Require/MissingRequireStatement.
- .get_upload_retries(params) ⇒ Object
-
.get_upload_timeout(params) ⇒ Object
rubocop:enable Require/MissingRequireStatement.
- .is_supported?(platform) ⇒ Boolean
- .output ⇒ Object
- .parse_test_device_string(td_string) ⇒ Object
- .platform_from_app_id(app_id) ⇒ Object
- .poll_test_finished(alpha_client, release_tests) ⇒ Object
- .poll_upload_release_operation(client, operation, binary_type) ⇒ Object
- .release_notes(params) ⇒ Object
- .release_version(release) ⇒ Object
- .run(params) ⇒ Object
- .test_password_from_params(params) ⇒ Object
- .test_release(alpha_client, release, test_devices, test_cases, username = nil, password = nil, username_resource = nil, password_resource = nil) ⇒ Object
- .update_release(client, release) ⇒ Object
- .upload_binary(client, app_name, binary_path, binary_type, timeout, retries) ⇒ Object
- .validate_aab_setup!(aab_info) ⇒ Object
Methods included from Fastlane::Auth::FirebaseAppDistributionAuthClient
Methods included from Helper::FirebaseAppDistributionHelper
app_id_from_params, app_name_from_app_id, binary_type_from_path, blank?, deep_symbolize_keys, get_ios_app_id_from_archive_plist, get_ios_app_id_from_plist, get_value_from_value_or_file, group_name, init_google_api_client, lane_platform, parse_plist, present?, project_name, project_number_from_app_id, string_to_array, xcode_archive_path
Class Method Details
.aab_certs_included?(test_certificate) ⇒ Boolean
215 216 217 218 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 215 def self.aab_certs_included?(test_certificate) !test_certificate.nil? && present?(test_certificate.hash_md5) && present?(test_certificate.hash_sha1) && present?(test_certificate.hash_sha256) end |
.aab_info_name(app_name) ⇒ Object
220 221 222 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 220 def self.aab_info_name(app_name) "#{app_name}/aabInfo" end |
.authors ⇒ Object
132 133 134 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 132 def self. ["Stefan Natchev", "Manny Jimenez Github: mannyjimenez0810, Alonso Salas Infante Github: alonsosalasinfante"] end |
.available_options ⇒ Object
454 455 456 457 458 459 460 461 462 463 464 465 466 467 468 469 470 471 472 473 474 475 476 477 478 479 480 481 482 483 484 485 486 487 488 489 490 491 492 493 494 495 496 497 498 499 500 501 502 503 504 505 506 507 508 509 510 511 512 513 514 515 516 517 518 519 520 521 522 523 524 525 526 527 528 529 530 531 532 533 534 535 536 537 538 539 540 541 542 543 544 545 546 547 548 549 550 551 552 553 554 555 556 557 558 559 560 561 562 563 564 565 566 567 568 569 570 571 572 573 574 575 576 577 578 579 580 581 582 583 584 585 586 587 588 589 590 591 592 593 594 595 596 597 598 599 600 601 602 603 604 605 606 607 608 609 610 611 612 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 454 def self. [ # iOS Specific FastlaneCore::ConfigItem.new(key: :ipa_path, env_name: "FIREBASEAPPDISTRO_IPA_PATH", description: "Path to your IPA file. Optional if you use the _gym_ or _xcodebuild_ action", optional: true), FastlaneCore::ConfigItem.new(key: :googleservice_info_plist_path, env_name: "GOOGLESERVICE_INFO_PLIST_PATH", description: "Path to your GoogleService-Info.plist file, relative to the archived product path (or directly, if no archived product path is found)", default_value: "GoogleService-Info.plist", optional: true, type: String), # Android Specific FastlaneCore::ConfigItem.new(key: :apk_path, env_name: "FIREBASEAPPDISTRO_APK_PATH", description: "Path to your APK file", optional: true), FastlaneCore::ConfigItem.new(key: :android_artifact_path, env_name: "FIREBASEAPPDISTRO_ANDROID_ARTIFACT_PATH", description: "Path to your APK or AAB file", optional: true), FastlaneCore::ConfigItem.new(key: :android_artifact_type, env_name: "FIREBASEAPPDISTRO_ANDROID_ARTIFACT_TYPE", description: "Android artifact type. Set to 'APK' or 'AAB'. Defaults to 'APK' if not set", default_value: "APK", default_value_dynamic: true, optional: true, verify_block: proc do |value| UI.user_error!("firebase_app_distribution: '#{value}' is not a valid value for android_artifact_type. Should be 'APK' or 'AAB'") unless ['APK', 'AAB'].include?(value) end), # General FastlaneCore::ConfigItem.new(key: :app, env_name: "FIREBASEAPPDISTRO_APP", description: "Your app's Firebase App ID. You can find the App ID in the Firebase console, on the General Settings page", optional: true, type: String), FastlaneCore::ConfigItem.new(key: :firebase_cli_path, deprecated: "This plugin no longer uses the Firebase CLI", env_name: "FIREBASEAPPDISTRO_FIREBASE_CLI_PATH", description: "Absolute path of the Firebase CLI command", type: String), FastlaneCore::ConfigItem.new(key: :debug, description: "Print verbose debug output", optional: true, default_value: false, type: Boolean), # Release Distribution FastlaneCore::ConfigItem.new(key: :upload_timeout, description: "Amount of seconds before the upload will timeout, if not completed", optional: true, default_value: DEFAULT_UPLOAD_TIMEOUT_SECONDS, type: Integer), FastlaneCore::ConfigItem.new(key: :upload_retries, description: "Maximum number of times the upload will retry, if not completed", optional: true, default_value: DEFAULT_UPLOAD_RETRIES, type: Integer), FastlaneCore::ConfigItem.new(key: :groups, env_name: "FIREBASEAPPDISTRO_GROUPS", description: "Group aliases used for distribution, separated by commas", optional: true, type: String), FastlaneCore::ConfigItem.new(key: :groups_file, env_name: "FIREBASEAPPDISTRO_GROUPS_FILE", description: "Path to file containing group aliases used for distribution, separated by commas or newlines", optional: true, type: String), FastlaneCore::ConfigItem.new(key: :testers, env_name: "FIREBASEAPPDISTRO_TESTERS", description: "Email addresses of testers, separated by commas", optional: true, type: String), FastlaneCore::ConfigItem.new(key: :testers_file, env_name: "FIREBASEAPPDISTRO_TESTERS_FILE", description: "Path to file containing email addresses of testers, separated by commas or newlines", optional: true, type: String), FastlaneCore::ConfigItem.new(key: :release_notes, env_name: "FIREBASEAPPDISTRO_RELEASE_NOTES", description: "Release notes for this build", optional: true, type: String), FastlaneCore::ConfigItem.new(key: :release_notes_file, env_name: "FIREBASEAPPDISTRO_RELEASE_NOTES_FILE", description: "Path to file containing release notes for this build", optional: true, type: String), # Release Testing FastlaneCore::ConfigItem.new(key: :test_devices, env_name: "FIREBASEAPPDISTRO_TEST_DEVICES", description: "List of devices (separated by semicolons) to run automated tests on, in the format 'model=<model-id>,version=<os-version-id>,locale=<locale>,orientation=<orientation>;model=<model-id>,...'. Run 'gcloud firebase test android|ios models list' to see available devices. Note: This feature is in beta", optional: true, type: String), FastlaneCore::ConfigItem.new(key: :test_devices_file, env_name: "FIREBASEAPPDISTRO_TEST_DEVICES_FILE", description: "Path to file containing a list of devices (sepatated by semicolons or newlines) to run automated tests on, in the format 'model=<model-id>,version=<os-version-id>,locale=<locale>,orientation=<orientation>;model=<model-id>,...'. " \ "Run 'gcloud firebase test android|ios models list' to see available devices. Note: This feature is in beta", optional: true, type: String), FastlaneCore::ConfigItem.new(key: :test_username, env_name: "FIREBASEAPPDISTRO_TEST_USERNAME", description: "Username for automatic login", optional: true, type: String), FastlaneCore::ConfigItem.new(key: :test_password, env_name: "FIREBASEAPPDISTRO_TEST_PASSWORD", description: "Password for automatic login. If using a real password consider using test_password_file or setting FIREBASEAPPDISTRO_TEST_PASSWORD to avoid exposing sensitive info", optional: true, type: String), FastlaneCore::ConfigItem.new(key: :test_password_file, env_name: "FIREBASEAPPDISTRO_TEST_PASSWORD_FILE", description: "Path to file containing password for automatic login", optional: true, type: String), FastlaneCore::ConfigItem.new(key: :test_username_resource, env_name: "FIREBASEAPPDISTRO_TEST_USERNAME_RESOURCE", description: "Resource name for the username field for automatic login", optional: true, type: String), FastlaneCore::ConfigItem.new(key: :test_password_resource, env_name: "FIREBASEAPPDISTRO_TEST_PASSWORD_RESOURCE", description: "Resource name for the password field for automatic login", optional: true, type: String), FastlaneCore::ConfigItem.new(key: :test_non_blocking, env_name: "FIREBASEAPPDISTRO_TEST_NON_BLOCKING", description: "Run automated tests without waiting for them to finish. Visit the Firebase console for the test results", optional: false, default_value: false, type: Boolean), FastlaneCore::ConfigItem.new(key: :test_case_ids, env_name: "FIREBASEAPPDISTRO_TEST_CASE_IDS", description: "Test Case IDs, separated by commas. Note: This feature is in beta", optional: true, type: String), FastlaneCore::ConfigItem.new(key: :test_case_ids_file, env_name: "FIREBASEAPPDISTRO_TEST_CASE_IDS_FILE", description: "Path to file with containing Test Case IDs, separated by commas or newlines. Note: This feature is in beta", optional: true, type: String), # Auth FastlaneCore::ConfigItem.new(key: :firebase_cli_token, description: "Auth token generated using the Firebase CLI's login:ci command", optional: true, type: String), FastlaneCore::ConfigItem.new(key: :service_credentials_file, description: "Path to Google service account json file", optional: true, type: String), FastlaneCore::ConfigItem.new(key: :service_credentials_json_data, description: "Google service account json file content", optional: true, type: String) ] end |
.create_release_test(alpha_client, release_name, device_executions, login_credential, test_case_name = nil) ⇒ Object
388 389 390 391 392 393 394 395 396 397 398 399 400 401 402 403 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 388 def self.create_release_test(alpha_client, release_name, device_executions, login_credential, test_case_name = nil) release_test = Google::Apis::FirebaseappdistributionV1alpha::GoogleFirebaseAppdistroV1alphaReleaseTest.new( device_executions: device_executions, login_credential: login_credential, test_case: test_case_name ) alpha_client.create_project_app_release_test(release_name, release_test) rescue Google::Apis::Error => err case err.status_code.to_i when 404 UI.user_error!("Test Case #{test_case_name} not found") else UI.crash!(err) end end |
.description ⇒ Object
128 129 130 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 128 def self.description "Release your beta builds with Firebase App Distribution" end |
.details ⇒ Object
supports markdown.
137 138 139 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 137 def self.details "Release your beta builds with Firebase App Distribution" end |
.device_to_s(device) ⇒ Object
450 451 452 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 450 def self.device_to_s(device) "#{device.model} (#{device.version}/#{device.orientation}/#{device.locale})" end |
.distribute_release(client, release, request) ⇒ Object
320 321 322 323 324 325 326 327 328 329 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 320 def self.distribute_release(client, release, request) client.distribute_project_app_release(release.name, request) rescue Google::Apis::Error => err case err.status_code.to_i when 400 UI.user_error!("#{ErrorMessage::INVALID_TESTERS}\nEmails: #{request.tester_emails} \nGroup Aliases: #{request.group_aliases}") else UI.crash!(err) end end |
.example_code ⇒ Object
618 619 620 621 622 623 624 625 626 627 628 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 618 def self.example_code [ <<-CODE firebase_app_distribution( app: "<your Firebase app ID>", testers: "snatchev@google.com, rebeccahe@google.com", test_devices: "model=shiba,version=34,locale=en,orientation=portrait;model=b0q,version=33,locale=en,orientation=portrait", ) CODE ] end |
.extract_release(operation) ⇒ Object
284 285 286 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 284 def self.extract_release(operation) Google::Apis::FirebaseappdistributionV1::GoogleFirebaseAppdistroV1Release.from_json(operation.response['release'].to_json) end |
.get_aab_info(client, app_name) ⇒ Object
298 299 300 301 302 303 304 305 306 307 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 298 def self.get_aab_info(client, app_name) client.get_project_app_aab_info(aab_info_name(app_name)) rescue Google::Apis::Error => err case err.status_code.to_i when 404 UI.user_error!(ErrorMessage::INVALID_APP_ID) else UI.crash!(err) end end |
.get_binary_path(platform, params) ⇒ Object
rubocop:disable Require/MissingRequireStatement
156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 156 def self.get_binary_path(platform, params) if platform == :ios return params[:ipa_path] || Actions.lane_context[SharedValues::IPA_OUTPUT_PATH] || Dir["*.ipa"].sort_by { |x| File.mtime(x) }.last end if platform == :android return params[:apk_path] || params[:android_artifact_path] if params[:apk_path] || params[:android_artifact_path] if params[:android_artifact_type] == 'AAB' return Actions.lane_context[SharedValues::GRADLE_AAB_OUTPUT_PATH] || Dir["*.aab"].last || Dir[File.join("app", "build", "outputs", "bundle", "release", "app-release.aab")].last end return Actions.lane_context[SharedValues::GRADLE_APK_OUTPUT_PATH] || Dir["*.apk"].last || Dir[File.join("app", "build", "outputs", "apk", "release", "app-release.apk")].last end UI.error("Unable to determine binary path for unsupported platform #{platform}.") nil end |
.get_upload_retries(params) ⇒ Object
190 191 192 193 194 195 196 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 190 def self.get_upload_retries(params) if params[:upload_retries] return params[:upload_retries] else return DEFAULT_UPLOAD_RETRIES end end |
.get_upload_timeout(params) ⇒ Object
rubocop:enable Require/MissingRequireStatement
182 183 184 185 186 187 188 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 182 def self.get_upload_timeout(params) if params[:upload_timeout] return params[:upload_timeout] else return DEFAULT_UPLOAD_TIMEOUT_SECONDS end end |
.is_supported?(platform) ⇒ Boolean
614 615 616 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 614 def self.is_supported?(platform) [:ios, :android].include?(platform) end |
.output ⇒ Object
630 631 632 633 634 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 630 def self.output [ ['FIREBASE_APP_DISTRO_RELEASE', 'A hash representing the uploaded release created in Firebase App Distribution'] ] end |
.parse_test_device_string(td_string) ⇒ Object
438 439 440 441 442 443 444 445 446 447 448 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 438 def self.parse_test_device_string(td_string) allowed_keys = %w[model version locale orientation] key_value_pairs = td_string.split(',').map do |key_value_string| key, value = key_value_string.split('=') unless allowed_keys.include?(key) UI.user_error!("Unrecognized key in test_devices. Can only contain keys #{allowed_keys.join(', ')}.") end [key, value] end Hash[key_value_pairs] end |
.platform_from_app_id(app_id) ⇒ Object
147 148 149 150 151 152 153 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 147 def self.platform_from_app_id(app_id) if app_id.include?(':ios:') :ios elsif app_id.include?(':android:') :android end end |
.poll_test_finished(alpha_client, release_tests) ⇒ Object
405 406 407 408 409 410 411 412 413 414 415 416 417 418 419 420 421 422 423 424 425 426 427 428 429 430 431 432 433 434 435 436 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 405 def self.poll_test_finished(alpha_client, release_tests) release_test_names = release_tests.map(&:name) TEST_MAX_POLLING_RETRIES.times do UI.("⏳ #{release_test_names.size} automated test results are pending.") sleep(TEST_POLLING_INTERVAL_SECONDS) release_test_names.delete_if do |release_test_name| release_test = alpha_client.get_project_app_release_test(release_test_name) if release_test.device_executions.all? { |e| e.state == 'PASSED' } true else release_test.device_executions.each do |de| case de.state when 'PASSED', 'IN_PROGRESS' next when 'FAILED' UI.test_failure!("Automated test failed for #{device_to_s(de.device)}: #{de.failed_reason}.") when 'INCONCLUSIVE' UI.test_failure!("Automated test inconclusive for #{device_to_s(de.device)}: #{de.inconclusive_reason}.") else UI.test_failure!("Unsupported automated test state for #{device_to_s(de.device)}: #{de.state}.") end end false end end if release_test_names.empty? UI.success("✅ Passed automated test(s).") return end end UI.test_failure!("It took longer than expected to process your test, please try again.") end |
.poll_upload_release_operation(client, operation, binary_type) ⇒ Object
232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 232 def self.poll_upload_release_operation(client, operation, binary_type) UPLOAD_MAX_POLLING_RETRIES.times do sleep(UPLOAD_POLLING_INTERVAL_SECONDS) operation = client.get_project_app_release_operation(operation.name) if operation.done && operation.response && operation.response['release'] release = extract_release(operation) case operation.response['result'] when 'RELEASE_UPDATED' UI.success("✅ Uploaded #{binary_type} successfully; updated provisioning profile of existing release #{release_version(release)}.") when 'RELEASE_UNMODIFIED' UI.success("✅ The same #{binary_type} was found in release #{release_version(release)} with no changes, skipping.") else UI.success("✅ Uploaded #{binary_type} successfully and created release #{release_version(release)}.") end break elsif !operation.done next else if operation.error && operation.error. UI.user_error!("#{ErrorMessage.upload_binary_error(binary_type)}: #{operation.error.}") else UI.user_error!(ErrorMessage.upload_binary_error(binary_type)) end end end unless operation.done && operation.response && operation.response['release'] UI.crash!("It took longer than expected to process your #{binary_type}, please try again.") end extract_release(operation) end |
.release_notes(params) ⇒ Object
224 225 226 227 228 229 230 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 224 def self.release_notes(params) release_notes_param = get_value_from_value_or_file(params[:release_notes], params[:release_notes_file]) # rubocop:disable Require/MissingRequireStatement release_notes_param || Actions.lane_context[SharedValues::FL_CHANGELOG] # rubocop:enable Require/MissingRequireStatement end |
.release_version(release) ⇒ Object
288 289 290 291 292 293 294 295 296 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 288 def self.release_version(release) if release.display_version && release.build_version "#{release.display_version} (#{release.build_version})" elsif release.display_version release.display_version else release.build_version end end |
.run(params) ⇒ Object
30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 30 def self.run(params) params.values # to validate all inputs before looking for the ipa/apk/aab app_id = app_id_from_params(params) app_name = app_name_from_app_id(app_id) platform = lane_platform || platform_from_app_id(app_id) timeout = get_upload_timeout(params) retries = get_upload_retries(params) binary_path = get_binary_path(platform, params) UI.user_error!("Couldn't determine path for #{platform} binary.") if binary_path.nil? UI.user_error!("Couldn't find #{platform} binary at path #{binary_path}.") unless File.exist?(binary_path) binary_type = binary_type_from_path(binary_path) # TODO(lkellogg): This sets the send timeout for all POST requests made by the client, but # ideally the timeout should only apply to the binary upload init_google_api_client(params[:debug], timeout) = (params[:service_credentials_file], params[:firebase_cli_token], params[:service_credentials_json_data], params[:debug]) client = Google::Apis::FirebaseappdistributionV1::FirebaseAppDistributionService.new client. = alpha_client = Google::Apis::FirebaseappdistributionV1alpha::FirebaseAppDistributionService.new alpha_client. = # If binary is an AAB, get the AAB info for this app, which includes the integration state # and certificate data if binary_type == :AAB aab_info = get_aab_info(client, app_name) validate_aab_setup!(aab_info) end binary_type = binary_type_from_path(binary_path) UI.("📡 Uploading the #{binary_type}.") operation = upload_binary(client, app_name, binary_path, binary_type, timeout, retries) UI.("🕵️ Validating upload…") release = poll_upload_release_operation(client, operation, binary_type) if binary_type == :AAB && aab_info && !aab_certs_included?(aab_info.test_certificate) updated_aab_info = get_aab_info(client, app_name) if aab_certs_included?(updated_aab_info.test_certificate) UI.("After you upload an AAB for the first time, App Distribution " \ "generates a new test certificate. All AAB uploads are re-signed with this test " \ "certificate. Use the certificate fingerprints below to register your app " \ "signing key with API providers, such as Google Sign-In and Google Maps.\n" \ "MD-1 certificate fingerprint: #{updated_aab_info.test_certificate.hash_md5}\n" \ "SHA-1 certificate fingerprint: #{updated_aab_info.test_certificate.hash_sha1}\n" \ "SHA-256 certificate fingerprint: #{updated_aab_info.test_certificate.hash_sha256}") end end release_notes = release_notes(params) if release_notes.nil? || release_notes.empty? UI.("⏩ No release notes passed in. Skipping this step.") else release.release_notes = Google::Apis::FirebaseappdistributionV1::GoogleFirebaseAppdistroV1ReleaseNotes.new( text: release_notes ) UI.("📜 Setting release notes.") release = update_release(client, release) end test_devices = get_value_from_value_or_file(params[:test_devices], params[:test_devices_file]) if present?(test_devices) test_cases = string_to_array(get_value_from_value_or_file(params[:test_case_ids], params[:test_case_ids_file]))&.map { |id| "#{app_name}/testCases/#{id}" } test_password = test_password_from_params(params) release_tests = test_release(alpha_client, release, test_devices, test_cases, params[:test_username], test_password, params[:test_username_resource], params[:test_password_resource]) unless params[:test_non_blocking] poll_test_finished(alpha_client, release_tests) end end testers = get_value_from_value_or_file(params[:testers], params[:testers_file]) groups = get_value_from_value_or_file(params[:groups], params[:groups_file]) emails = string_to_array(testers) group_aliases = string_to_array(groups) if present?(emails) || present?(group_aliases) request = Google::Apis::FirebaseappdistributionV1::GoogleFirebaseAppdistroV1DistributeReleaseRequest.new( tester_emails: emails, group_aliases: group_aliases ) UI.("📦 Distributing release.") distribute_release(client, release, request) else UI.("⏩ No testers or groups passed in. Skipping this step.") end UI.success("🎉 App Distribution upload finished successfully. Setting Actions.lane_context[SharedValues::FIREBASE_APP_DISTRO_RELEASE] to the uploaded release.") UI.("🔗 View this release in the Firebase console: #{release.firebase_console_uri}") if release.firebase_console_uri UI.("🔗 Share this release with testers who have access: #{release.testing_uri}") if release.testing_uri UI.("🔗 Download the release binary (link expires in 1 hour): #{release.binary_download_uri}") if release.binary_download_uri release_hash = deep_symbolize_keys(JSON.parse(release.to_json)) Actions.lane_context[SharedValues::FIREBASE_APP_DISTRO_RELEASE] = release_hash release_hash end |
.test_password_from_params(params) ⇒ Object
141 142 143 144 145 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 141 def self.test_password_from_params(params) test_password = get_value_from_value_or_file(params[:test_password], params[:test_password_file]) # Remove trailing newline if present test_password && test_password.sub(/\r?\n$/, "") end |
.test_release(alpha_client, release, test_devices, test_cases, username = nil, password = nil, username_resource = nil, password_resource = nil) ⇒ Object
331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 331 def self.test_release(alpha_client, release, test_devices, test_cases, username = nil, password = nil, username_resource = nil, password_resource = nil) if present?(test_cases) && (!username_resource.nil? || !password_resource.nil?) UI.user_error!("Password and username resource names are not supported for the testing agent.") end if username_resource.nil? ^ password_resource.nil? UI.user_error!("Username and password resource names for automated tests need to be specified together.") end field_hints = nil if !username_resource.nil? && !password_resource.nil? field_hints = Google::Apis::FirebaseappdistributionV1alpha::GoogleFirebaseAppdistroV1alphaLoginCredentialFieldHints.new( username_resource_name: username_resource, password_resource_name: password_resource ) end if username.nil? ^ password.nil? UI.user_error!("Username and password for automated tests need to be specified together.") end login_credential = nil if !username.nil? && !password.nil? login_credential = Google::Apis::FirebaseappdistributionV1alpha::GoogleFirebaseAppdistroV1alphaLoginCredential.new( username: username, password: password, field_hints: field_hints ) else unless field_hints.nil? UI.user_error!("Must specify username and password for automated tests if resource names are set.") end end device_executions = string_to_array(test_devices, /[;\n]/).map do |td_string| td_hash = parse_test_device_string(td_string) Google::Apis::FirebaseappdistributionV1alpha::GoogleFirebaseAppdistroV1alphaDeviceExecution.new( device: Google::Apis::FirebaseappdistributionV1alpha::GoogleFirebaseAppdistroV1alphaTestDevice.new( model: td_hash['model'], version: td_hash['version'], orientation: td_hash['orientation'], locale: td_hash['locale'] ) ) end UI.("🤖 Starting automated tests. Note: This feature is in beta.") release_tests = [] if present?(test_cases) test_cases.each do |tc| release_tests.push(create_release_test(alpha_client, release.name, device_executions, login_credential, tc)) end else release_tests.push(create_release_test(alpha_client, release.name, device_executions, login_credential)) end release_tests end |
.update_release(client, release) ⇒ Object
309 310 311 312 313 314 315 316 317 318 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 309 def self.update_release(client, release) client.patch_project_app_release(release.name, release) rescue Google::Apis::Error => err case err.status_code.to_i when 400 UI.user_error!("#{ErrorMessage::INVALID_RELEASE_NOTES}: #{err.body}") else UI.crash!(err) end end |
.upload_binary(client, app_name, binary_path, binary_type, timeout, retries) ⇒ Object
265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 265 def self.upload_binary(client, app_name, binary_path, binary_type, timeout, retries) = Google::Apis::RequestOptions.new(max_elapsed_time: timeout, retries: retries) .header = { 'Content-Type' => 'application/octet-stream', 'X-Goog-Upload-File-Name' => CGI.escape(File.basename(binary_path)) } begin client.upload_medium(app_name, File.open(binary_path, 'rb'), upload_source: binary_path, options: ) rescue Google::Apis::Error => err case err.status_code.to_i when 403 UI.crash!(ErrorMessage::PERMISSION_DENIED_ERROR) else UI.crash!("#{ErrorMessage.upload_binary_error(binary_type)} (#{err}, status_code: #{err.status_code})") end end end |
.validate_aab_setup!(aab_info) ⇒ Object
198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 |
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 198 def self.validate_aab_setup!(aab_info) if aab_info && aab_info.integration_state != 'INTEGRATED' && aab_info.integration_state != 'AAB_STATE_UNAVAILABLE' case aab_info.integration_state when 'PLAY_ACCOUNT_NOT_LINKED' UI.user_error!(ErrorMessage::PLAY_ACCOUNT_NOT_LINKED) when 'APP_NOT_PUBLISHED' UI.user_error!(ErrorMessage::APP_NOT_PUBLISHED) when 'NO_APP_WITH_GIVEN_BUNDLE_ID_IN_PLAY_ACCOUNT' UI.user_error!(ErrorMessage::NO_APP_WITH_GIVEN_BUNDLE_ID_IN_PLAY_ACCOUNT) when 'PLAY_IAS_TERMS_NOT_ACCEPTED' UI.user_error!(ErrorMessage::PLAY_IAS_TERMS_NOT_ACCEPTED) else UI.user_error!(ErrorMessage.aab_upload_error(aab_info.integration_state)) end end end |