Class: Fastlane::Actions::FirebaseAppDistributionAction

Inherits:
Action
  • Object
show all
Extended by:
Fastlane::Auth::FirebaseAppDistributionAuthClient, Helper::FirebaseAppDistributionHelper
Defined in:
lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb

Overview

rubocop:disable Metrics/ClassLength

Constant Summary collapse

DEFAULT_UPLOAD_TIMEOUT_SECONDS =
300
UPLOAD_MAX_POLLING_RETRIES =
60
UPLOAD_POLLING_INTERVAL_SECONDS =
5
TEST_MAX_POLLING_RETRIES =
40
TEST_POLLING_INTERVAL_SECONDS =
30

Constants included from Fastlane::Auth::FirebaseAppDistributionAuthClient

Fastlane::Auth::FirebaseAppDistributionAuthClient::CLIENT_ID, Fastlane::Auth::FirebaseAppDistributionAuthClient::CLIENT_SECRET, Fastlane::Auth::FirebaseAppDistributionAuthClient::REDACTION_CHARACTER, Fastlane::Auth::FirebaseAppDistributionAuthClient::REDACTION_EXPOSED_LENGTH, Fastlane::Auth::FirebaseAppDistributionAuthClient::SCOPE, Fastlane::Auth::FirebaseAppDistributionAuthClient::TOKEN_CREDENTIAL_URI

Class Method Summary collapse

Methods included from Fastlane::Auth::FirebaseAppDistributionAuthClient

get_authorization

Methods included from Helper::FirebaseAppDistributionHelper

app_id_from_params, app_name_from_app_id, binary_type_from_path, blank?, deep_symbolize_keys, get_ios_app_id_from_archive_plist, get_ios_app_id_from_plist, get_value_from_value_or_file, group_name, init_google_api_client, lane_platform, parse_plist, present?, project_name, project_number_from_app_id, string_to_array, xcode_archive_path

Class Method Details

.aab_certs_included?(test_certificate) ⇒ Boolean

Returns:

  • (Boolean)


205
206
207
208
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 205

def self.aab_certs_included?(test_certificate)
  !test_certificate.nil? && present?(test_certificate.hash_md5) && present?(test_certificate.hash_sha1) &&
    present?(test_certificate.hash_sha256)
end

.aab_info_name(app_name) ⇒ Object



210
211
212
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 210

def self.aab_info_name(app_name)
  "#{app_name}/aabInfo"
end

.authorsObject



130
131
132
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 130

def self.authors
  ["Stefan Natchev", "Manny Jimenez Github: mannyjimenez0810, Alonso Salas Infante Github: alonsosalasinfante"]
end

.available_optionsObject



458
459
460
461
462
463
464
465
466
467
468
469
470
471
472
473
474
475
476
477
478
479
480
481
482
483
484
485
486
487
488
489
490
491
492
493
494
495
496
497
498
499
500
501
502
503
504
505
506
507
508
509
510
511
512
513
514
515
516
517
518
519
520
521
522
523
524
525
526
527
528
529
530
531
532
533
534
535
536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
553
554
555
556
557
558
559
560
561
562
563
564
565
566
567
568
569
570
571
572
573
574
575
576
577
578
579
580
581
582
583
584
585
586
587
588
589
590
591
592
593
594
595
596
597
598
599
600
601
602
603
604
605
606
607
608
609
610
611
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 458

def self.available_options
  [
    # iOS Specific
    FastlaneCore::ConfigItem.new(key: :ipa_path,
                                 env_name: "FIREBASEAPPDISTRO_IPA_PATH",
                                 description: "Path to your IPA file. Optional if you use the _gym_ or _xcodebuild_ action",
                                 optional: true),
    FastlaneCore::ConfigItem.new(key: :googleservice_info_plist_path,
                                 env_name: "GOOGLESERVICE_INFO_PLIST_PATH",
                                 description: "Path to your GoogleService-Info.plist file, relative to the archived product path (or directly, if no archived product path is found)",
                                 default_value: "GoogleService-Info.plist",
                                 optional: true,
                                 type: String),
    # Android Specific
    FastlaneCore::ConfigItem.new(key: :apk_path,
                                 env_name: "FIREBASEAPPDISTRO_APK_PATH",
                                 description: "Path to your APK file",
                                 optional: true),
    FastlaneCore::ConfigItem.new(key: :android_artifact_path,
                                 env_name: "FIREBASEAPPDISTRO_ANDROID_ARTIFACT_PATH",
                                 description: "Path to your APK or AAB file",
                                 optional: true),
    FastlaneCore::ConfigItem.new(key: :android_artifact_type,
                                 env_name: "FIREBASEAPPDISTRO_ANDROID_ARTIFACT_TYPE",
                                 description: "Android artifact type. Set to 'APK' or 'AAB'. Defaults to 'APK' if not set",
                                 default_value: "APK",
                                 default_value_dynamic: true,
                                 optional: true,
                                 verify_block: proc do |value|
                                   UI.user_error!("firebase_app_distribution: '#{value}' is not a valid value for android_artifact_type. Should be 'APK' or 'AAB'") unless ['APK', 'AAB'].include?(value)
                                 end),
    # General
    FastlaneCore::ConfigItem.new(key: :app,
                                 env_name: "FIREBASEAPPDISTRO_APP",
                                 description: "Your app's Firebase App ID. You can find the App ID in the Firebase console, on the General Settings page",
                                 optional: true,
                                 type: String),
    FastlaneCore::ConfigItem.new(key: :firebase_cli_path,
                                 deprecated: "This plugin no longer uses the Firebase CLI",
                                 env_name: "FIREBASEAPPDISTRO_FIREBASE_CLI_PATH",
                                 description: "Absolute path of the Firebase CLI command",
                                 type: String),
    FastlaneCore::ConfigItem.new(key: :debug,
                                description: "Print verbose debug output",
                                optional: true,
                                default_value: false,
                                type: Boolean),

    # Release Distribution
    FastlaneCore::ConfigItem.new(key: :upload_timeout,
                                 description: "Amount of seconds before the upload will  timeout, if not completed",
                                 optional: true,
                                 default_value: DEFAULT_UPLOAD_TIMEOUT_SECONDS,
                                 type: Integer),
    FastlaneCore::ConfigItem.new(key: :groups,
                                 env_name: "FIREBASEAPPDISTRO_GROUPS",
                                 description: "Group aliases used for distribution, separated by commas",
                                 optional: true,
                                 type: String),
    FastlaneCore::ConfigItem.new(key: :groups_file,
                                 env_name: "FIREBASEAPPDISTRO_GROUPS_FILE",
                                 description: "Path to file containing group aliases used for distribution, separated by commas or newlines",
                                 optional: true,
                                 type: String),
    FastlaneCore::ConfigItem.new(key: :testers,
                                 env_name: "FIREBASEAPPDISTRO_TESTERS",
                                 description: "Email addresses of testers, separated by commas",
                                 optional: true,
                                 type: String),
    FastlaneCore::ConfigItem.new(key: :testers_file,
                                 env_name: "FIREBASEAPPDISTRO_TESTERS_FILE",
                                 description: "Path to file containing email addresses of testers, separated by commas or newlines",
                                 optional: true,
                                 type: String),
    FastlaneCore::ConfigItem.new(key: :release_notes,
                                 env_name: "FIREBASEAPPDISTRO_RELEASE_NOTES",
                                 description: "Release notes for this build",
                                 optional: true,
                                 type: String),
    FastlaneCore::ConfigItem.new(key: :release_notes_file,
                                 env_name: "FIREBASEAPPDISTRO_RELEASE_NOTES_FILE",
                                 description: "Path to file containing release notes for this build",
                                 optional: true,
                                 type: String),

    # Release Testing
    FastlaneCore::ConfigItem.new(key: :test_devices,
                                 env_name: "FIREBASEAPPDISTRO_TEST_DEVICES",
                                 description: "List of devices (separated by semicolons) to run automated tests on, in the format 'model=<model-id>,version=<os-version-id>,locale=<locale>,orientation=<orientation>;model=<model-id>,...'. Run 'gcloud firebase test android|ios models list' to see available devices. Note: This feature is in beta",
                                 optional: true,
                                 type: String),
    FastlaneCore::ConfigItem.new(key: :test_devices_file,
                                 env_name: "FIREBASEAPPDISTRO_TEST_DEVICES_FILE",
                                 description: "Path to file containing a list of devices (sepatated by semicolons or newlines) to run automated tests on, in the format 'model=<model-id>,version=<os-version-id>,locale=<locale>,orientation=<orientation>;model=<model-id>,...'. " \
                                 "Run 'gcloud firebase test android|ios models list' to see available devices. Note: This feature is in beta",
                                 optional: true,
                                 type: String),
    FastlaneCore::ConfigItem.new(key: :test_username,
                                 env_name: "FIREBASEAPPDISTRO_TEST_USERNAME",
                                 description: "Username for automatic login",
                                 optional: true,
                                 type: String),
    FastlaneCore::ConfigItem.new(key: :test_password,
                                 env_name: "FIREBASEAPPDISTRO_TEST_PASSWORD",
                                 description: "Password for automatic login. If using a real password consider using test_password_file or setting FIREBASEAPPDISTRO_TEST_PASSWORD to avoid exposing sensitive info",
                                 optional: true,
                                 type: String),
    FastlaneCore::ConfigItem.new(key: :test_password_file,
                                 env_name: "FIREBASEAPPDISTRO_TEST_PASSWORD_FILE",
                                description: "Path to file containing password for automatic login",
                                optional: true,
                                type: String),
    FastlaneCore::ConfigItem.new(key: :test_username_resource,
                                 env_name: "FIREBASEAPPDISTRO_TEST_USERNAME_RESOURCE",
                                 description: "Resource name for the username field for automatic login",
                                 optional: true,
                                 type: String),
    FastlaneCore::ConfigItem.new(key: :test_password_resource,
                                 env_name: "FIREBASEAPPDISTRO_TEST_PASSWORD_RESOURCE",
                                 description: "Resource name for the password field for automatic login",
                                 optional: true,
                                 type: String),
    FastlaneCore::ConfigItem.new(key: :test_non_blocking,
                                 env_name: "FIREBASEAPPDISTRO_TEST_NON_BLOCKING",
                                 description: "Run automated tests without waiting for them to finish. Visit the Firebase console for the test results",
                                 optional: false,
                                 default_value: false,
                                 type: Boolean),
    FastlaneCore::ConfigItem.new(key: :test_case_ids,
                                 env_name: "FIREBASEAPPDISTRO_TEST_CASE_IDS",
                                 description: "Test Case IDs, separated by commas. Note: This feature is in beta",
                                 optional: true,
                                 type: String),
    FastlaneCore::ConfigItem.new(key: :test_case_ids_file,
                                 env_name: "FIREBASEAPPDISTRO_TEST_CASE_IDS_FILE",
                                 description: "Path to file with containing Test Case IDs, separated by commas or newlines. Note: This feature is in beta",
                                 optional: true,
                                 type: String),

    # Auth
    FastlaneCore::ConfigItem.new(key: :firebase_cli_token,
                                 description: "Auth token generated using the Firebase CLI's login:ci command",
                                 optional: true,
                                 type: String),
    FastlaneCore::ConfigItem.new(key: :service_credentials_file,
                                 description: "Path to Google service account json file",
                                 optional: true,
                                 type: String),
    FastlaneCore::ConfigItem.new(key: :service_credentials_json_data,
                                 description: "Google service account json file content",
                                 optional: true,
                                 type: String)
  ]
end

.create_release_test(alpha_client, release_name, device_executions, login_credential, test_case_name = nil) ⇒ Object



392
393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 392

def self.create_release_test(alpha_client, release_name, device_executions, , test_case_name = nil)
  release_test =
    Google::Apis::FirebaseappdistributionV1alpha::GoogleFirebaseAppdistroV1alphaReleaseTest.new(
      device_executions: device_executions,
      login_credential: ,
      test_case: test_case_name
    )
  alpha_client.create_project_app_release_test(release_name, release_test)
rescue Google::Apis::Error => err
  case err.status_code.to_i
  when 404
    UI.user_error!("Test Case #{test_case_name} not found")
  else
    UI.crash!(err)
  end
end

.descriptionObject



126
127
128
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 126

def self.description
  "Release your beta builds with Firebase App Distribution"
end

.detailsObject

supports markdown.



135
136
137
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 135

def self.details
  "Release your beta builds with Firebase App Distribution"
end

.device_to_s(device) ⇒ Object



454
455
456
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 454

def self.device_to_s(device)
  "#{device.model} (#{device.version}/#{device.orientation}/#{device.locale})"
end

.distribute_release(client, release, request) ⇒ Object



324
325
326
327
328
329
330
331
332
333
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 324

def self.distribute_release(client, release, request)
  client.distribute_project_app_release(release.name, request)
rescue Google::Apis::Error => err
  case err.status_code.to_i
  when 400
    UI.user_error!("#{ErrorMessage::INVALID_TESTERS}\nEmails: #{request.tester_emails} \nGroup Aliases: #{request.group_aliases}")
  else
    UI.crash!(err)
  end
end

.example_codeObject



617
618
619
620
621
622
623
624
625
626
627
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 617

def self.example_code
  [
    "      firebase_app_distribution(\n        app: \"<your Firebase app ID>\",\n        testers: \"[email protected], [email protected]\",\n        test_devices: \"model=shiba,version=34,locale=en,orientation=portrait;model=b0q,version=33,locale=en,orientation=portrait\",\n      )\n    CODE\n  ]\nend\n"

.extract_release(operation) ⇒ Object



288
289
290
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 288

def self.extract_release(operation)
  Google::Apis::FirebaseappdistributionV1::GoogleFirebaseAppdistroV1Release.from_json(operation.response['release'].to_json)
end

.get_aab_info(client, app_name) ⇒ Object



302
303
304
305
306
307
308
309
310
311
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 302

def self.get_aab_info(client, app_name)
  client.get_project_app_aab_info(aab_info_name(app_name))
rescue Google::Apis::Error => err
  case err.status_code.to_i
  when 404
    UI.user_error!(ErrorMessage::INVALID_APP_ID)
  else
    UI.crash!(err)
  end
end

.get_binary_path(platform, params) ⇒ Object

rubocop:disable Require/MissingRequireStatement



154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 154

def self.get_binary_path(platform, params)
  if platform == :ios
    return params[:ipa_path] ||
           Actions.lane_context[SharedValues::IPA_OUTPUT_PATH] ||
           Dir["*.ipa"].sort_by { |x| File.mtime(x) }.last
  end

  if platform == :android
    return params[:apk_path] || params[:android_artifact_path] if params[:apk_path] || params[:android_artifact_path]

    if params[:android_artifact_type] == 'AAB'
      return Actions.lane_context[SharedValues::GRADLE_AAB_OUTPUT_PATH] ||
             Dir["*.aab"].last ||
             Dir[File.join("app", "build", "outputs", "bundle", "release", "app-release.aab")].last
    end

    return Actions.lane_context[SharedValues::GRADLE_APK_OUTPUT_PATH] ||
           Dir["*.apk"].last ||
           Dir[File.join("app", "build", "outputs", "apk", "release", "app-release.apk")].last
  end

  UI.error("Unable to determine binary path for unsupported platform #{platform}.")
  nil
end

.get_upload_timeout(params) ⇒ Object

rubocop:enable Require/MissingRequireStatement



180
181
182
183
184
185
186
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 180

def self.get_upload_timeout(params)
  if params[:upload_timeout]
    return params[:upload_timeout]
  else
    return DEFAULT_UPLOAD_TIMEOUT_SECONDS
  end
end

.is_supported?(platform) ⇒ Boolean

Returns:

  • (Boolean)


613
614
615
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 613

def self.is_supported?(platform)
  [:ios, :android].include?(platform)
end

.outputObject



629
630
631
632
633
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 629

def self.output
  [
    ['FIREBASE_APP_DISTRO_RELEASE', 'A hash representing the uploaded release created in Firebase App Distribution']
  ]
end

.parse_test_device_string(td_string) ⇒ Object



442
443
444
445
446
447
448
449
450
451
452
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 442

def self.parse_test_device_string(td_string)
  allowed_keys = %w[model version locale orientation]
  key_value_pairs = td_string.split(',').map do |key_value_string|
    key, value = key_value_string.split('=')
    unless allowed_keys.include?(key)
      UI.user_error!("Unrecognized key in test_devices. Can only contain keys #{allowed_keys.join(', ')}.")
    end
    [key, value]
  end
  Hash[key_value_pairs]
end

.platform_from_app_id(app_id) ⇒ Object



145
146
147
148
149
150
151
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 145

def self.platform_from_app_id(app_id)
  if app_id.include?(':ios:')
    :ios
  elsif app_id.include?(':android:')
    :android
  end
end

.poll_test_finished(alpha_client, release_tests) ⇒ Object



409
410
411
412
413
414
415
416
417
418
419
420
421
422
423
424
425
426
427
428
429
430
431
432
433
434
435
436
437
438
439
440
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 409

def self.poll_test_finished(alpha_client, release_tests)
  release_test_names = release_tests.map(&:name)
  TEST_MAX_POLLING_RETRIES.times do
    UI.message("⏳ #{release_test_names.size} automated test results are pending.")
    sleep(TEST_POLLING_INTERVAL_SECONDS)
    release_test_names.delete_if do |release_test_name|
      release_test = alpha_client.get_project_app_release_test(release_test_name)
      if release_test.device_executions.all? { |e| e.state == 'PASSED' }
        true
      else
        release_test.device_executions.each do |de|
          case de.state
          when 'PASSED', 'IN_PROGRESS'
            next
          when 'FAILED'
            UI.test_failure!("Automated test failed for #{device_to_s(de.device)}: #{de.failed_reason}.")
          when 'INCONCLUSIVE'
            UI.test_failure!("Automated test inconclusive for #{device_to_s(de.device)}: #{de.inconclusive_reason}.")
          else
            UI.test_failure!("Unsupported automated test state for #{device_to_s(de.device)}: #{de.state}.")
          end
        end
        false
      end
    end
    if release_test_names.empty?
      UI.success("✅ Passed automated test(s).")
      return
    end
  end
  UI.test_failure!("It took longer than expected to process your test, please try again.")
end

.poll_upload_release_operation(client, operation, binary_type) ⇒ Object



222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 222

def self.poll_upload_release_operation(client, operation, binary_type)
  UPLOAD_MAX_POLLING_RETRIES.times do
    sleep(UPLOAD_POLLING_INTERVAL_SECONDS)
    operation = client.get_project_app_release_operation(operation.name)
    if operation.done && operation.response && operation.response['release']
      release = extract_release(operation)
      case operation.response['result']
      when 'RELEASE_UPDATED'
        UI.success("✅ Uploaded #{binary_type} successfully; updated provisioning profile of existing release #{release_version(release)}.")
      when 'RELEASE_UNMODIFIED'
        UI.success("✅ The same #{binary_type} was found in release #{release_version(release)} with no changes, skipping.")
      else
        UI.success("✅ Uploaded #{binary_type} successfully and created release #{release_version(release)}.")
      end
      break
    elsif !operation.done
      next
    else
      if operation.error && operation.error.message
        UI.user_error!("#{ErrorMessage.upload_binary_error(binary_type)}: #{operation.error.message}")
      else
        UI.user_error!(ErrorMessage.upload_binary_error(binary_type))
      end
    end
  end

  unless operation.done && operation.response && operation.response['release']
    UI.crash!("It took longer than expected to process your #{binary_type}, please try again.")
  end

  extract_release(operation)
end

.release_notes(params) ⇒ Object



214
215
216
217
218
219
220
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 214

def self.release_notes(params)
  release_notes_param =
    get_value_from_value_or_file(params[:release_notes], params[:release_notes_file])
  # rubocop:disable Require/MissingRequireStatement
  release_notes_param || Actions.lane_context[SharedValues::FL_CHANGELOG]
  # rubocop:enable Require/MissingRequireStatement
end

.release_version(release) ⇒ Object



292
293
294
295
296
297
298
299
300
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 292

def self.release_version(release)
  if release.display_version && release.build_version
    "#{release.display_version} (#{release.build_version})"
  elsif release.display_version
    release.display_version
  else
    release.build_version
  end
end

.run(params) ⇒ Object



29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 29

def self.run(params)
  params.values # to validate all inputs before looking for the ipa/apk/aab

  app_id = app_id_from_params(params)
  app_name = app_name_from_app_id(app_id)
  platform = lane_platform || platform_from_app_id(app_id)
  timeout = get_upload_timeout(params)

  binary_path = get_binary_path(platform, params)
  UI.user_error!("Couldn't determine path for #{platform} binary.") if binary_path.nil?
  UI.user_error!("Couldn't find #{platform} binary at path #{binary_path}.") unless File.exist?(binary_path)
  binary_type = binary_type_from_path(binary_path)

  # TODO(lkellogg): This sets the send timeout for all POST requests made by the client, but
  #     ideally the timeout should only apply to the binary upload
  init_google_api_client(params[:debug], timeout)
  authorization = get_authorization(params[:service_credentials_file], params[:firebase_cli_token], params[:service_credentials_json_data], params[:debug])
  client = Google::Apis::FirebaseappdistributionV1::FirebaseAppDistributionService.new
  client.authorization = authorization
  alpha_client = Google::Apis::FirebaseappdistributionV1alpha::FirebaseAppDistributionService.new
  alpha_client.authorization = authorization

  # If binary is an AAB, get the AAB info for this app, which includes the integration state
  # and certificate data
  if binary_type == :AAB
    aab_info = get_aab_info(client, app_name)
    validate_aab_setup!(aab_info)
  end

  binary_type = binary_type_from_path(binary_path)
  UI.message("📡 Uploading the #{binary_type}.")
  operation = upload_binary(client, app_name, binary_path, binary_type, timeout)
  UI.message("🕵️ Validating upload…")
  release = poll_upload_release_operation(client, operation, binary_type)

  if binary_type == :AAB && aab_info && !aab_certs_included?(aab_info.test_certificate)
    updated_aab_info = get_aab_info(client, app_name)
    if aab_certs_included?(updated_aab_info.test_certificate)
      UI.message("After you upload an AAB for the first time, App Distribution " \
        "generates a new test certificate. All AAB uploads are re-signed with this test " \
        "certificate. Use the certificate fingerprints below to register your app " \
        "signing key with API providers, such as Google Sign-In and Google Maps.\n" \
        "MD-1 certificate fingerprint: #{updated_aab_info.test_certificate.hash_md5}\n" \
        "SHA-1 certificate fingerprint: #{updated_aab_info.test_certificate.hash_sha1}\n" \
        "SHA-256 certificate fingerprint: #{updated_aab_info.test_certificate.hash_sha256}")
    end
  end

  release_notes = release_notes(params)
  if release_notes.nil? || release_notes.empty?
    UI.message("⏩ No release notes passed in. Skipping this step.")
  else
    release.release_notes = Google::Apis::FirebaseappdistributionV1::GoogleFirebaseAppdistroV1ReleaseNotes.new(
      text: release_notes
    )
    UI.message("📜 Setting release notes.")
    release = update_release(client, release)
  end

  test_devices =
    get_value_from_value_or_file(params[:test_devices], params[:test_devices_file])
  if present?(test_devices)
    test_cases =
      string_to_array(get_value_from_value_or_file(params[:test_case_ids], params[:test_case_ids_file]))&.map { |id| "#{app_name}/testCases/#{id}" }
    test_password = test_password_from_params(params)
    release_tests = test_release(alpha_client, release, test_devices, test_cases, params[:test_username], test_password, params[:test_username_resource], params[:test_password_resource])
    unless params[:test_non_blocking]
      poll_test_finished(alpha_client, release_tests)
    end
  end

  testers = get_value_from_value_or_file(params[:testers], params[:testers_file])
  groups = get_value_from_value_or_file(params[:groups], params[:groups_file])
  emails = string_to_array(testers)
  group_aliases = string_to_array(groups)
  if present?(emails) || present?(group_aliases)
    request = Google::Apis::FirebaseappdistributionV1::GoogleFirebaseAppdistroV1DistributeReleaseRequest.new(
      tester_emails: emails,
      group_aliases: group_aliases
    )
    UI.message("📦 Distributing release.")
    distribute_release(client, release, request)
  else
    UI.message("⏩ No testers or groups passed in. Skipping this step.")
  end

  UI.success("🎉 App Distribution upload finished successfully. Setting Actions.lane_context[SharedValues::FIREBASE_APP_DISTRO_RELEASE] to the uploaded release.")

  UI.message("🔗 View this release in the Firebase console: #{release.firebase_console_uri}") if release.firebase_console_uri
  UI.message("🔗 Share this release with testers who have access: #{release.testing_uri}") if release.testing_uri
  UI.message("🔗 Download the release binary (link expires in 1 hour): #{release.binary_download_uri}") if release.binary_download_uri

  release_hash = deep_symbolize_keys(JSON.parse(release.to_json))
  Actions.lane_context[SharedValues::FIREBASE_APP_DISTRO_RELEASE] = release_hash
  release_hash
end

.test_password_from_params(params) ⇒ Object



139
140
141
142
143
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 139

def self.test_password_from_params(params)
  test_password = get_value_from_value_or_file(params[:test_password], params[:test_password_file])
  # Remove trailing newline if present
  test_password && test_password.sub(/\r?\n$/, "")
end

.test_release(alpha_client, release, test_devices, test_cases, username = nil, password = nil, username_resource = nil, password_resource = nil) ⇒ Object



335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 335

def self.test_release(alpha_client, release, test_devices, test_cases, username = nil, password = nil, username_resource = nil, password_resource = nil)
  if present?(test_cases) && (!username_resource.nil? || !password_resource.nil?)
    UI.user_error!("Password and username resource names are not supported for the testing agent.")
  end
  if username_resource.nil? ^ password_resource.nil?
    UI.user_error!("Username and password resource names for automated tests need to be specified together.")
  end
  field_hints = nil
  if !username_resource.nil? && !password_resource.nil?
    field_hints =
      Google::Apis::FirebaseappdistributionV1alpha::GoogleFirebaseAppdistroV1alphaLoginCredentialFieldHints.new(
        username_resource_name: username_resource,
        password_resource_name: password_resource
      )
  end

  if username.nil? ^ password.nil?
    UI.user_error!("Username and password for automated tests need to be specified together.")
  end
   = nil
  if !username.nil? && !password.nil?
     =
      Google::Apis::FirebaseappdistributionV1alpha::GoogleFirebaseAppdistroV1alphaLoginCredential.new(
        username: username,
        password: password,
        field_hints: field_hints
      )
  else
    unless field_hints.nil?
      UI.user_error!("Must specify username and password for automated tests if resource names are set.")
    end
  end

  device_executions = string_to_array(test_devices, /[;\n]/).map do |td_string|
    td_hash = parse_test_device_string(td_string)
    Google::Apis::FirebaseappdistributionV1alpha::GoogleFirebaseAppdistroV1alphaDeviceExecution.new(
      device: Google::Apis::FirebaseappdistributionV1alpha::GoogleFirebaseAppdistroV1alphaTestDevice.new(
        model: td_hash['model'],
        version: td_hash['version'],
        orientation: td_hash['orientation'],
        locale: td_hash['locale']
      )
    )
  end

  UI.message("🤖 Starting automated tests. Note: This feature is in beta.")
  release_tests = []
  if present?(test_cases)
    test_cases.each do |tc|
      release_tests.push(create_release_test(alpha_client, release.name, device_executions, , tc))
    end
  else
    release_tests.push(create_release_test(alpha_client, release.name, device_executions, ))
  end
  release_tests
end

.update_release(client, release) ⇒ Object



313
314
315
316
317
318
319
320
321
322
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 313

def self.update_release(client, release)
  client.patch_project_app_release(release.name, release)
rescue Google::Apis::Error => err
  case err.status_code.to_i
  when 400
    UI.user_error!("#{ErrorMessage::INVALID_RELEASE_NOTES}: #{err.body}")
  else
    UI.crash!(err)
  end
end

.upload_binary(client, app_name, binary_path, binary_type, timeout) ⇒ Object



255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 255

def self.upload_binary(client, app_name, binary_path, binary_type, timeout)
  options = Google::Apis::RequestOptions.new
  options.max_elapsed_time = timeout # includes retries (default = no retries)
  options.header = {
    'Content-Type' => 'application/octet-stream',
    'X-Goog-Upload-File-Name' => CGI.escape(File.basename(binary_path)),
    'X-Goog-Upload-Protocol' => 'raw'
  }

  # For some reason calling the client.upload_medium returns nil when
  # it should return a long running operation object, so we make a
  # standard http call instead and convert it to a long running object
  # https://github.com/googleapis/google-api-ruby-client/blob/main/generated/google-apis-firebaseappdistribution_v1/lib/google/apis/firebaseappdistribution_v1/service.rb#L79
  # TODO(kbolay): Prefer client.upload_medium
  response = begin
    client.http(
      :post,
      "https://firebaseappdistribution.googleapis.com/upload/v1/#{app_name}/releases:upload",
      body: File.open(binary_path, 'rb'),
      options: options
    )
  rescue Google::Apis::Error => err
    case err.status_code.to_i
    when 403
      UI.crash!(ErrorMessage::PERMISSION_DENIED_ERROR)
    else
      UI.crash!("#{ErrorMessage.upload_binary_error(binary_type)} (#{err}, status_code: #{err.status_code})")
    end
  end

  Google::Apis::FirebaseappdistributionV1::GoogleLongrunningOperation.from_json(response)
end

.validate_aab_setup!(aab_info) ⇒ Object



188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
# File 'lib/fastlane/plugin/firebase_app_distribution/actions/firebase_app_distribution_action.rb', line 188

def self.validate_aab_setup!(aab_info)
  if aab_info && aab_info.integration_state != 'INTEGRATED' && aab_info.integration_state != 'AAB_STATE_UNAVAILABLE'
    case aab_info.integration_state
    when 'PLAY_ACCOUNT_NOT_LINKED'
      UI.user_error!(ErrorMessage::)
    when 'APP_NOT_PUBLISHED'
      UI.user_error!(ErrorMessage::APP_NOT_PUBLISHED)
    when 'NO_APP_WITH_GIVEN_BUNDLE_ID_IN_PLAY_ACCOUNT'
      UI.user_error!(ErrorMessage::)
    when 'PLAY_IAS_TERMS_NOT_ACCEPTED'
      UI.user_error!(ErrorMessage::PLAY_IAS_TERMS_NOT_ACCEPTED)
    else
      UI.user_error!(ErrorMessage.aab_upload_error(aab_info.integration_state))
    end
  end
end